//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _PARALLELPORT_
#define _PARALLELPORT_

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _IOCARD_
   #include "iocard.hpp"
#endif

//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO ParallelPort : public IOCard
{
   typedef IOCard Inherited;

public:


   ParallelPort ( const char * inName = NULL ); // for LPT1

   ParallelPort ( const unsigned short inBaseIOPort );
   virtual ~ParallelPort ();

   unsigned short baseIOPort () const
      {return theBaseIOPort;}

protected:

   enum ParallelPortConstants
   {
       maxNumberOfPoints = 17
     , statusOffset = 1
     , controlOffset = 2
   };

   virtual IOCard & updateInputs ( const IOTrack & inTrack
                                 , long * inImage
                                 , long & inTrackState
                                 );

   virtual IOCard & updateOutputs ( const IOTrack & inTrack
                                  , const long * const inImage
                                 , long & inTrackState
                                  );

   // Tracks can have mask of only
   //    "OOOOOOOOOOOOIIIII"
   // or "OOOOOOOOIIIIIIIII"
   // each point corresponds to following pins # on the LPT port:
   // 2 3 4 5 6 7 8 9 1 14 16 17 15 13 12 10 11
   virtual IOCard & addTrack ( IOTrack & inTrack ) ;

private:

   unsigned short theBaseIOPort; 

}; // end ParallelPort class declaration

}; // end namespace

#endif