//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _PCIAC28CARD_
#define _PCIAC28CARD_

#ifndef _UNIIO_H
   #include "uniio.h"
#endif

#ifndef _AC28CARD_
   #include "ac28card.hpp"
#endif

//-----------------------------------------------------------------------------
namespace Uniio
{

class _Export_UNIIO PciAC28Card : public AC28Card
{
   typedef IOCard Inherited;

public:

   PciAC28Card ( const bool inResetLevel
               , const unsigned inCardId = 0
               );

   virtual ~PciAC28Card ();

   enum PCIAC28Constants
   {
       maxNumberOfCards = 32  
     , LED1OffLED2Off   = 0x00
     , LED1OnLED2Off    = 0x01
     , LED1OffLED2On    = 0x02
     , LED1OnLED2On     = 0x03
     , SetResetAssertHighOffset = 0x0F
     , SetResetAssertHigh       = 0x0F
     , SetResetAssertLow        = 0xFF
     , AssertResetOffset        = 0x3F
     , AssertReset              = 0x01
     , NegateReset              = 0x00
   };

   PciAC28Card & setLEDs (const unsigned inValue);


protected:

   virtual char readBank (const unsigned inBank) const;
   virtual void writeBank (const unsigned inBank, const char inData);

private:
   static unsigned baseAddress(const unsigned inCardId);
   static unsigned resetAddress(const unsigned inCardId);

   unsigned theLEDsAddress;

}; // end PciAC28Card class declaration


}; // end namespace

#endif