//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB BBDSoft ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _OSIFACE_
   #include "osiface.hpp"
#endif

#ifndef _WINDOWS_
   #define WIN32_LEAN_AND_MEAN
   #include <windows.h>
#endif

#include <rtapi.h>


#ifndef _STDEXCEPT_
   #include <stdexcept>
#endif


using namespace std;
using namespace Uniio;


OSInterface * OSInterface::theInterface = NULL;

//-----------------------------------------------------------------------------
OSInterface & OSInterface::osinterface ()
{
BFUNCTRACE_DEVELOP ();

//If it is first call to the function - create Operating system interface object
if (!theInterface)
{
   theInterface = new OSInterface;
} // endif

return * theInterface;
} // end OSInterface::osinterface()


//-----------------------------------------------------------------------------
OSInterface::OSInterface ()
{
BFUNCTRACE_DEVELOP ();

} // end constructor


//-----------------------------------------------------------------------------
OSInterface::~OSInterface ()
{
BFUNCTRACE_DEVELOP ();

} // end destructor


//-----------------------------------------------------------------------------
void * OSInterface::mapMemory ( unsigned long inAddress
                              , unsigned long inBlockSize
                              , bool fWriteAccess
                              )
{
BFUNCTRACE_DEVELOP ();

LARGE_INTEGER theAddr;
theAddr.HighPart = 0;
theAddr.LowPart = inAddress;

void * virtualAddress = RtMapMemory ( theAddr
                                    , inBlockSize
                                    , false
                                    );
if (!virtualAddress)
{
   runtime_error exc ("OS Interface: can not get pointer to physical memory block.");
   throw exc;
} // endif

return virtualAddress;
} // end OSInterface::mapMemory (..)


//-----------------------------------------------------------------------------
OSInterface & OSInterface::unmapMemory ( void * inVirtualAddress )
{
BFUNCTRACE_DEVELOP ();

if (!inVirtualAddress)
{
   return * this;
} // endif

if ( !RtUnmapMemory (inVirtualAddress) )
{
   runtime_error exc ("OS Interface: can not release physical memory block.");
   throw exc;
} // endif

return * this;
} // end OSInterface::unmapMemory (..)


//-----------------------------------------------------------------------------
void OSInterface::sleep ( const unsigned long inMicroseconds )
{
BFUNCTRACE_DEVELOP ();

LARGE_INTEGER theInt;
theInt.HighPart = 0;
theInt.LowPart = inMicroseconds * 10 ; // sleep time should be provided in 100ns'
RtSleepFt (&theInt);
} // end OSInterface::sleep (..)


//-----------------------------------------------------------------------------
OSInterface & OSInterface::enablePortIO ( const unsigned inPortNumber
                                        , bool fWriteAccess
                                        )
{
BFUNCTRACE_DEVELOP ();

// RtEnablePortIo should not be called for already enabled ports
if (theDisabledPortSet.find (inPortNumber) == theDisabledPortSet.end())
{
   theDisabledPortSet.insert ( inPortNumber );

   if (!RtEnablePortIo ( (PUCHAR)inPortNumber , 1 ) )
   {
      runtime_error exc ("OS Interface: RtEnablePortIo failed.");
      throw exc;
   } // endif
} // endif

theDisabledPortSet.insert ( inPortNumber );

return * this;
} // end OSInterface::enablePortIO (..)


//-----------------------------------------------------------------------------
OSInterface & OSInterface::disablePortIO ( const unsigned inPortNumber )
{
BFUNCTRACE_DEVELOP ();

PortSet::iterator i = theDisabledPortSet.find (inPortNumber);
if ( i != theDisabledPortSet.end())
{
   theDisabledPortSet.erase (i);
} // endif

// RtEnablePortIo should not be called if another client is using the port
if (theDisabledPortSet.find (inPortNumber) == theDisabledPortSet.end())
{
   if (!RtDisablePortIo ( (PUCHAR)inPortNumber , 1 ) )
   {
      runtime_error exc ("OS Interface: RtDisablePortIo failed.");
      throw exc;
   } // endif
} // endif
return * this;
} // end OSInterface::disablePortIO (..)
