//=============================================================================
//  Unified IO - C++ interface for industrial IO cards
//  Copyright (C) 2000, UAB "BBD SOFT" ( http://www.bbdsoft.com/ )
//
// This material is provided "as is", with absolutely no warranty expressed
// or implied. Any use is at your own risk.
//
// Permission to use or copy this software for any purpose is hereby granted 
// without fee, provided the above notices are retained on all copies.
// Permission to modify the code and to distribute modified code is granted,
// provided the above notices are retained, and a notice that the code was
// modified is included with the above copyright notice.
//
//  The author of this program may be contacted at developers@bbdsoft.com
//=============================================================================


#ifndef _BTRACE_
   #include "btrace.hpp"
#endif

#ifndef _PCICFGRTX_
   #include "pcicfgrtx.hpp"
#endif

#include <cstddef>
#include <stdexcept>

#ifndef _WINDOWS_
   #define WIN32_LEAN_AND_MEAN
   #include <windows.h>
#endif

#include <rtapi.h>

using namespace Uniio;
using namespace std;

//-----------------------------------------------------------------------------
PCIConfigRTX::PCIConfigRTX ()
   : PCIConfig ()
{
BFUNCTRACE_DEVELOP ();
} // end constructor


//-----------------------------------------------------------------------------
PCIConfigRTX::~PCIConfigRTX ()
{
BFUNCTRACE_DEVELOP ();
} // end destructor

//-----------------------------------------------------------------------------
bool PCIConfigRTX::exists() const
{
BFUNCTRACE_DEVELOP ();
   return true;
}

//-----------------------------------------------------------------------------
void
PCIConfigRTX::scanBus ()
{
BFUNCTRACE_DEVELOP ();
   if (!exists())
       throw runtime_error ("PCI bus scanning by this Config type is not possible");

   bool isBusExists = true;
   for ( unsigned long aBusNumber = 0
       ; isBusExists
       ; aBusNumber++
       )
   {
      for ( unsigned long aSlotNumber = 0
          ; aSlotNumber < 0xFF
          ; aSlotNumber++
          )
      {
         unsigned long * aBuffer = new unsigned long [64];
         unsigned long aResult
           = RtGetBusDataByOffset ( PCIConfiguration
                                  , aBusNumber
                                  , aSlotNumber
                                  , &aBuffer
                                  , 0 // offset
                                  , PCI_COMMON_HDR_LENGTH
                                  );
         if (aResult == 0)
         {
            // Bus does not exists lets finish
            isBusExists = false;
            break;
         }
         if (aResult == 2)
         {
            // there is no device in this slot
            continue;
         }
         // insert this device
         addDevice (aBusNumber, aSlotNumber, aBuffer);
      }
   } // end for aBusNumber
} // end 


//-----------------------------------------------------------------------------
unsigned char
PCIConfigRTX::readChar ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset ) const
{
BFUNCTRACE_DEVELOP ();

   unsigned char aValue;
   RtGetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
   return aValue;
}

//-----------------------------------------------------------------------------
unsigned short
PCIConfigRTX::readShort ( const unsigned char inBus
                      , const unsigned char inSlot
                      , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();

   unsigned short aValue;
   RtGetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
   return aValue;
}

//-----------------------------------------------------------------------------
unsigned long
PCIConfigRTX::readLong ( const unsigned char inBus
                     , const unsigned char inSlot
                     , const unsigned char inOffset) const
{
BFUNCTRACE_DEVELOP ();
   unsigned long aValue;
   RtGetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
   return aValue;
}

//-----------------------------------------------------------------------------
void
PCIConfigRTX::writeChar ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned char inValue)
{
BFUNCTRACE_DEVELOP ();

   unsigned char aValue (inValue);
   RtSetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
}

//-----------------------------------------------------------------------------
void
PCIConfigRTX::writeShort ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned short inValue)
{
BFUNCTRACE_DEVELOP ();
   unsigned short aValue (inValue);
   RtSetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
}

//-----------------------------------------------------------------------------
void
PCIConfigRTX::writeLong ( const unsigned char inBus
                  , const unsigned char inSlot
                  , const unsigned char inOffset
                  , const unsigned long inValue)
{
BFUNCTRACE_DEVELOP ();
   unsigned long aValue (inValue);
   RtSetBusDataByOffset ( PCIConfiguration
                                  , inBus
                                  , inSlot
                                  , &aValue
                                  , inOffset
                                  , sizeof(aValue)
                                  );
}
